#include <exec/types.h>
#include <exec/nodes.h>
#include <exec/lists.h>
#include <exec/memory.h>
#include <utility/tagitem.h>
#include <powerup/ppclib/interface.h>
#include <powerup/ppclib/message.h>
#define __SASC
#include <powerup/gcclib/powerup_protos.h>
#undef __SASC

#define TEXT    "Text sent by PPC processor\n"

BPTR    MyFile;
void printf(char *String);


int main(void)
{
  struct TagItem  MyTags[10];
  void            *PPCPort;
  void            *ReplyPort;
  void            *M68kPort;
  void            *PPCMsg;
  void            *M68kMsg;
  void            *Body;
  ULONG           result;

#ifdef DEBUG
  if (MyFile = PPCOpen("con:0/0/640/200/MessageDemo - PPC output/CLOSE", MODE_NEWFILE))
  {
    printf("Creating message port\n");
#endif
    MyTags[0].ti_Tag  = PPCPORTTAG_NAME;
    MyTags[0].ti_Data = (ULONG) "PPC port";
    MyTags[1].ti_Tag  = TAG_DONE;
    if (PPCPort = PPCCreatePort(MyTags))
    {
#ifdef DEBUG
      printf("Waiting for M68k message\n");
#endif
      PPCWaitPort(PPCPort);

#ifdef DEBUG
      printf("Getting message\n");
#endif
      if (M68kMsg = PPCGetMessage(PPCPort))
      {
#ifdef DEBUG
        printf("Message: ");
        printf((char*) PPCGetMessageAttr(M68kMsg, PPCMSGTAG_DATA));
#endif
        PPCReplyMessage(M68kMsg);
      }
      else
      {
#ifdef DEBUG
        printf("Did not get m68k msg\n");
#endif
      }

#ifdef DEBUG
      printf("Allocating memory for message body\n");
#endif
      if (Body = PPCAllocVec(sizeof(TEXT), MEMF_PUBLIC))
      {
#ifdef DEBUG
        printf("Creating reply port\n");
#endif
        MyTags[0].ti_Tag = TAG_DONE;
        if (ReplyPort = PPCCreatePort(MyTags))
        {
#ifdef DEBUG
          printf("Creating message\n");
#endif
          if (PPCMsg = PPCCreateMessage(ReplyPort, sizeof(TEXT)))
          {
#ifdef DEBUG
            printf("Obtaining M68k port\n");
#endif
            MyTags[0].ti_Tag  = PPCPORTTAG_NAME;
            MyTags[0].ti_Data = (ULONG) "M68k port";
            MyTags[1].ti_Tag  = TAG_DONE;
            while (!(M68kPort = PPCObtainPort(MyTags)));

#ifdef DEBUG
            printf("Sending message\n");
#endif
            strcpy(Body, TEXT);

            PPCSendMessage(M68kPort,
                           PPCMsg,
                           Body,
                           sizeof(TEXT),
                           0x87654321);

#ifdef DEBUG
            printf("Waiting for reply\n");
#endif
            PPCWaitPort(ReplyPort);

#ifdef DEBUG
            printf("Releasing M68k port\n");
#endif
            PPCReleasePort(M68kPort);

#ifdef DEBUG
            printf("Deleting message\n");
#endif
            PPCDeleteMessage(PPCMsg);

          }
          else
          {
#ifdef DEBUG
            printf("Could not create ppc msg\n");
#endif
          }

#ifdef DEBUG
          printf("Deleting reply port\n");
#endif
          while (PPCDeletePort(ReplyPort) == FALSE);

        }
        else
        {
#ifdef DEBUG
          printf("Could not create reply port\n");
#endif
        }

#ifdef DEBUG
        printf("Freeing message body memory\n");
#endif
        PPCFreeVec(Body);
      }
      else
      {
#ifdef DEBUG
        printf("Could not alloc mem for msg body\n");
#endif
      }

#ifdef DEBUG
      printf("Deleting message port\n");
#endif
      while (PPCDeletePort(PPCPort) == FALSE)
      {
/*
        PPCRawDoFmt("deleteport failed...\n",
                    NULL,
                    1,        // 0=Buffer,1=serial <> NOT supported yet
                    NULL);
*/
      }
    }
    else
    {
#ifdef DEBUG
      printf("Could not create ppc port\n");
#endif
    }

#ifdef DEBUG
/*    printf("Closing output\n");
    PPCClose(MyFile);*/
  }
#endif
}


void printf(char *String)
{
  PPCWrite(MyFile, String, strlen(String));
}
